module ynu_ti_kecore
(
 clk_in
,en_mk_in
,en_ek_in
,sel_ek_in
,sel_rcon_in
,mk_in
,rk1_in
,rk2_in
,rk3_in
,rk4_in
,k1_out
,k2_out
,k3_out
,k4_out
);

input           clk_in;
input           en_mk_in;
input           en_ek_in;
input           sel_ek_in;
input   [3:0]   sel_rcon_in;
input   [127:0] mk_in;
input   [127:0] rk1_in,rk2_in,rk3_in,rk4_in;
output  [127:0] k1_out,k2_out,k3_out,k4_out;


wire    [127:0] mk_o_w;
wire    [127:0] mk_o1_w,mk_o2_w,mk_o3_w,mk_o4_w;
wire    [127:0] ek_temp1_w,ek_temp2_w,ek_temp3_w,ek_temp4_w;
wire    [127:0] ek_i1_w,ek_i2_w,ek_i3_w,ek_i4_w;
wire    [127:0] ek_o1_w,ek_o2_w,ek_o3_w,ek_o4_w;
wire    [31:0]  ek0_o1_w,ek1_o1_w,ek2_o1_w,ek3_o1_w,
                ek0_o2_w,ek1_o2_w,ek2_o2_w,ek3_o2_w,
                ek0_o3_w,ek1_o3_w,ek2_o3_w,ek3_o3_w,
                ek0_o4_w,ek1_o4_w,ek2_o4_w,ek3_o4_w;
wire    [31:0]  rotword1_w,rotword2_w,rotword3_w,rotword4_w;
wire    [31:0]  subword1_w,subword2_w,subword3_w,subword4_w;
wire    [7:0]   rcon_w;
wire    [31:0]  temp1_w,temp2_w,temp3_w,temp4_w;
wire    [31:0]  xor0_o1_w,xor1_o1_w,xor2_o1_w,xor3_o1_w,
                xor0_o2_w,xor1_o2_w,xor2_o2_w,xor3_o2_w,
                xor0_o3_w,xor1_o3_w,xor2_o3_w,xor3_o3_w,
                xor0_o4_w,xor1_o4_w,xor2_o4_w,xor3_o4_w;


ynu_ti_ffe  #(128)  U_MK_REG
(
 .clk_in    ( clk_in    )
,.en_in     ( en_mk_in  )
,.d_in      ( mk_in     )
,.d_out     ( mk_o_w    )
);

ynu_ti_xor2 #(128)  U_RAND_XOR1 (.d0_in(mk_o_w), .d1_in(rk1_in), .d_out(mk_o1_w));
ynu_ti_xor2 #(128)  U_RAND_XOR2 (.d0_in(mk_o_w), .d1_in(rk2_in), .d_out(mk_o2_w));
ynu_ti_xor2 #(128)  U_RAND_XOR3 (.d0_in(mk_o_w), .d1_in(rk3_in), .d_out(mk_o3_w));
assign  mk_o4_w = rk4_in;

ynu_ti_mux2 #(128)  U_EK_SEL1
(
 .sel_in    ( sel_ek_in     )
,.d0_in     ( ek_temp1_w    )
,.d1_in     ( mk_o1_w       )
,.d_out     ( ek_i1_w       )
);

ynu_ti_mux2 #(128)  U_EK_SEL2
(
 .sel_in    ( sel_ek_in     )
,.d0_in     ( ek_temp2_w    )
,.d1_in     ( mk_o2_w       )
,.d_out     ( ek_i2_w       )
);

ynu_ti_mux2 #(128)  U_EK_SEL3
(
 .sel_in    ( sel_ek_in     )
,.d0_in     ( ek_temp3_w    )
,.d1_in     ( mk_o3_w       )
,.d_out     ( ek_i3_w       )
);

ynu_ti_mux2 #(128)  U_EK_SEL4
(
 .sel_in    ( sel_ek_in     )
,.d0_in     ( ek_temp4_w    )
,.d1_in     ( mk_o4_w       )
,.d_out     ( ek_i4_w       )
);

ynu_ti_ffe  #(128)  U_EK_REG1
(
 .clk_in    ( clk_in    )
,.en_in     ( en_ek_in  )
,.d_in      ( ek_i1_w   )
,.d_out     ( ek_o1_w   )
);

ynu_ti_ffe  #(128)  U_EK_REG2
(
 .clk_in    ( clk_in    )
,.en_in     ( en_ek_in  )
,.d_in      ( ek_i2_w   )
,.d_out     ( ek_o2_w   )
);

ynu_ti_ffe  #(128)  U_EK_REG3
(
 .clk_in    ( clk_in    )
,.en_in     ( en_ek_in  )
,.d_in      ( ek_i3_w   )
,.d_out     ( ek_o3_w   )
);

ynu_ti_ffe  #(128)  U_EK_REG4
(
 .clk_in    ( clk_in    )
,.en_in     ( en_ek_in  )
,.d_in      ( ek_i4_w   )
,.d_out     ( ek_o4_w   )
);


assign  {ek0_o1_w,ek1_o1_w,ek2_o1_w,ek3_o1_w} = ek_o1_w;
assign  {ek0_o2_w,ek1_o2_w,ek2_o2_w,ek3_o2_w} = ek_o2_w;
assign  {ek0_o3_w,ek1_o3_w,ek2_o3_w,ek3_o3_w} = ek_o3_w;
assign  {ek0_o4_w,ek1_o4_w,ek2_o4_w,ek3_o4_w} = ek_o4_w;

assign  rotword1_w = {ek3_o1_w[23:0],ek3_o1_w[31:24]};
assign  rotword2_w = {ek3_o2_w[23:0],ek3_o2_w[31:24]};
assign  rotword3_w = {ek3_o3_w[23:0],ek3_o3_w[31:24]};
assign  rotword4_w = {ek3_o4_w[23:0],ek3_o4_w[31:24]};

ynu_ti_subword_s4   U_SUBWORD
(
 .d1_in     ( rotword1_w    )
,.d2_in     ( rotword2_w    )
,.d3_in     ( rotword3_w    )
,.d4_in     ( rotword4_w    )
,.d1_out    ( subword1_w    )
,.d2_out    ( subword2_w    )
,.d3_out    ( subword3_w    )
,.d4_out    ( subword4_w    )
);

ynu_ti_rconsel  U_RCON_SEL
(
 .sel_in    ( sel_rcon_in   )
,.d_out     ( rcon_w        )
);

assign  temp1_w = subword1_w;

assign  temp2_w = subword2_w;

assign  temp3_w = subword3_w;

ynu_ti_xor2 #(8)    U_RCON_XOR  (.d0_in(subword4_w[31:24]), .d1_in(rcon_w), .d_out(temp4_w[31:24]));

assign  temp4_w[23:0] = subword4_w[23:0];

ynu_ti_xor2 #(32)   U_TEMP1_XOR0    (.d0_in(temp1_w)  , .d1_in(ek0_o1_w), .d_out(xor0_o1_w));
ynu_ti_xor2 #(32)   U_TEMP1_XOR1    (.d0_in(xor0_o1_w), .d1_in(ek1_o1_w), .d_out(xor1_o1_w));
ynu_ti_xor2 #(32)   U_TEMP1_XOR2    (.d0_in(xor1_o1_w), .d1_in(ek2_o1_w), .d_out(xor2_o1_w));
ynu_ti_xor2 #(32)   U_TEMP1_XOR3    (.d0_in(xor2_o1_w), .d1_in(ek3_o1_w), .d_out(xor3_o1_w));

ynu_ti_xor2 #(32)   U_TEMP2_XOR0    (.d0_in(temp2_w)  , .d1_in(ek0_o2_w), .d_out(xor0_o2_w));
ynu_ti_xor2 #(32)   U_TEMP2_XOR1    (.d0_in(xor0_o2_w), .d1_in(ek1_o2_w), .d_out(xor1_o2_w));
ynu_ti_xor2 #(32)   U_TEMP2_XOR2    (.d0_in(xor1_o2_w), .d1_in(ek2_o2_w), .d_out(xor2_o2_w));
ynu_ti_xor2 #(32)   U_TEMP2_XOR3    (.d0_in(xor2_o2_w), .d1_in(ek3_o2_w), .d_out(xor3_o2_w));

ynu_ti_xor2 #(32)   U_TEMP3_XOR0    (.d0_in(temp3_w)  , .d1_in(ek0_o3_w), .d_out(xor0_o3_w));
ynu_ti_xor2 #(32)   U_TEMP3_XOR1    (.d0_in(xor0_o3_w), .d1_in(ek1_o3_w), .d_out(xor1_o3_w));
ynu_ti_xor2 #(32)   U_TEMP3_XOR2    (.d0_in(xor1_o3_w), .d1_in(ek2_o3_w), .d_out(xor2_o3_w));
ynu_ti_xor2 #(32)   U_TEMP3_XOR3    (.d0_in(xor2_o3_w), .d1_in(ek3_o3_w), .d_out(xor3_o3_w));

ynu_ti_xor2 #(32)   U_TEMP4_XOR0    (.d0_in(temp4_w)  , .d1_in(ek0_o4_w), .d_out(xor0_o4_w));
ynu_ti_xor2 #(32)   U_TEMP4_XOR1    (.d0_in(xor0_o4_w), .d1_in(ek1_o4_w), .d_out(xor1_o4_w));
ynu_ti_xor2 #(32)   U_TEMP4_XOR2    (.d0_in(xor1_o4_w), .d1_in(ek2_o4_w), .d_out(xor2_o4_w));
ynu_ti_xor2 #(32)   U_TEMP4_XOR3    (.d0_in(xor2_o4_w), .d1_in(ek3_o4_w), .d_out(xor3_o4_w));

assign  ek_temp1_w = {xor0_o1_w,xor1_o1_w,xor2_o1_w,xor3_o1_w};
assign  ek_temp2_w = {xor0_o2_w,xor1_o2_w,xor2_o2_w,xor3_o2_w};
assign  ek_temp3_w = {xor0_o3_w,xor1_o3_w,xor2_o3_w,xor3_o3_w};
assign  ek_temp4_w = {xor0_o4_w,xor1_o4_w,xor2_o4_w,xor3_o4_w};

assign  k1_out = ek_i1_w;
assign  k2_out = ek_i2_w;
assign  k3_out = ek_i3_w;
assign  k4_out = ek_i4_w;


endmodule
