module ynu_ti_prng
(
 clk_in
,rst_n_in
,mode_in
,set_in
,en_in
,seed_in
,rk1_out
,rk2_out
,rk3_out
,rk4_out
,rd1_out
,rd2_out
,rd3_out
,rd4_out
);


input           clk_in;
input           rst_n_in;
input   [1:0]   mode_in;
input           set_in;
input           en_in;
input   [127:0] seed_in;
output  [127:0] rk1_out,rk2_out,rk3_out,rk4_out;
output  [127:0] rd1_out,rd2_out,rd3_out,rd4_out;

reg     [128:0] mrand_r;
wire            msb_w;
wire            en_w;
wire    [127:0] rdat_w;


// x^129 + x^5 + 1
always @(posedge clk_in or negedge rst_n_in)
begin
    if(rst_n_in==1'b0)
        mrand_r <= 129'h12b7e151628aed2a6abf7158809cf4f3c;
    else
        begin
            if(set_in==1'b1)
                mrand_r <= {1'b1,seed_in};
            else if(en_w==1'b1)
                mrand_r <= {msb_w,mrand_r[128:1]};
            else
                mrand_r <= mrand_r;
        end
end


assign  msb_w = mrand_r[0] ^ mrand_r[5];
assign  en_w = (mode_in==2'b00) ? 1'b1 : en_in;
assign  rdat_w = (mode_in[1]==1'b1) ? {128{mode_in[0]}} : mrand_r[127:0];

//bootstrap
assign  rk1_out = { rdat_w[089]
                   ,rdat_w[112]
                   ,rdat_w[070]
                   ,rdat_w[001]
                   ,rdat_w[116]
                   ,rdat_w[119]
                   ,rdat_w[091]
                   ,rdat_w[001]
                   ,rdat_w[069]
                   ,rdat_w[095]
                   ,rdat_w[069]
                   ,rdat_w[050]
                   ,rdat_w[124]
                   ,rdat_w[029]
                   ,rdat_w[123]
                   ,rdat_w[055]
                   ,rdat_w[079]
                   ,rdat_w[029]
                   ,rdat_w[122]
                   ,rdat_w[127]
                   ,rdat_w[034]
                   ,rdat_w[090]
                   ,rdat_w[032]
                   ,rdat_w[077]
                   ,rdat_w[084]
                   ,rdat_w[081]
                   ,rdat_w[105]
                   ,rdat_w[030]
                   ,rdat_w[102]
                   ,rdat_w[052]
                   ,rdat_w[002]
                   ,rdat_w[061]
                   ,rdat_w[061]
                   ,rdat_w[008]
                   ,rdat_w[022]
                   ,rdat_w[060]
                   ,rdat_w[127]
                   ,rdat_w[012]
                   ,rdat_w[015]
                   ,rdat_w[093]
                   ,rdat_w[125]
                   ,rdat_w[099]
                   ,rdat_w[089]
                   ,rdat_w[054]
                   ,rdat_w[058]
                   ,rdat_w[033]
                   ,rdat_w[048]
                   ,rdat_w[038]
                   ,rdat_w[126]
                   ,rdat_w[002]
                   ,rdat_w[062]
                   ,rdat_w[126]
                   ,rdat_w[006]
                   ,rdat_w[069]
                   ,rdat_w[084]
                   ,rdat_w[026]
                   ,rdat_w[079]
                   ,rdat_w[036]
                   ,rdat_w[088]
                   ,rdat_w[080]
                   ,rdat_w[003]
                   ,rdat_w[016]
                   ,rdat_w[063]
                   ,rdat_w[093]
                   ,rdat_w[103]
                   ,rdat_w[119]
                   ,rdat_w[039]
                   ,rdat_w[107]
                   ,rdat_w[127]
                   ,rdat_w[023]
                   ,rdat_w[084]
                   ,rdat_w[043]
                   ,rdat_w[124]
                   ,rdat_w[080]
                   ,rdat_w[086]
                   ,rdat_w[058]
                   ,rdat_w[030]
                   ,rdat_w[051]
                   ,rdat_w[021]
                   ,rdat_w[000]
                   ,rdat_w[112]
                   ,rdat_w[085]
                   ,rdat_w[044]
                   ,rdat_w[048]
                   ,rdat_w[024]
                   ,rdat_w[082]
                   ,rdat_w[024]
                   ,rdat_w[000]
                   ,rdat_w[096]
                   ,rdat_w[106]
                   ,rdat_w[032]
                   ,rdat_w[055]
                   ,rdat_w[059]
                   ,rdat_w[105]
                   ,rdat_w[076]
                   ,rdat_w[095]
                   ,rdat_w[014]
                   ,rdat_w[031]
                   ,rdat_w[105]
                   ,rdat_w[063]
                   ,rdat_w[072]
                   ,rdat_w[015]
                   ,rdat_w[099]
                   ,rdat_w[052]
                   ,rdat_w[081]
                   ,rdat_w[045]
                   ,rdat_w[084]
                   ,rdat_w[007]
                   ,rdat_w[060]
                   ,rdat_w[105]
                   ,rdat_w[055]
                   ,rdat_w[065]
                   ,rdat_w[070]
                   ,rdat_w[114]
                   ,rdat_w[117]
                   ,rdat_w[017]
                   ,rdat_w[045]
                   ,rdat_w[066]
                   ,rdat_w[076]
                   ,rdat_w[063]
                   ,rdat_w[003]
                   ,rdat_w[051]
                   ,rdat_w[081]
                   ,rdat_w[118]
                   ,rdat_w[115]
                   ,rdat_w[032]
                   ,rdat_w[083]
                   ,rdat_w[037] };

assign  rk2_out = { rdat_w[038]
                   ,rdat_w[070]
                   ,rdat_w[083]
                   ,rdat_w[025]
                   ,rdat_w[065]
                   ,rdat_w[008]
                   ,rdat_w[097]
                   ,rdat_w[120]
                   ,rdat_w[058]
                   ,rdat_w[096]
                   ,rdat_w[030]
                   ,rdat_w[035]
                   ,rdat_w[082]
                   ,rdat_w[090]
                   ,rdat_w[013]
                   ,rdat_w[111]
                   ,rdat_w[012]
                   ,rdat_w[116]
                   ,rdat_w[124]
                   ,rdat_w[040]
                   ,rdat_w[002]
                   ,rdat_w[124]
                   ,rdat_w[099]
                   ,rdat_w[001]
                   ,rdat_w[036]
                   ,rdat_w[048]
                   ,rdat_w[105]
                   ,rdat_w[039]
                   ,rdat_w[105]
                   ,rdat_w[057]
                   ,rdat_w[019]
                   ,rdat_w[056]
                   ,rdat_w[108]
                   ,rdat_w[062]
                   ,rdat_w[106]
                   ,rdat_w[079]
                   ,rdat_w[039]
                   ,rdat_w[041]
                   ,rdat_w[122]
                   ,rdat_w[083]
                   ,rdat_w[043]
                   ,rdat_w[118]
                   ,rdat_w[020]
                   ,rdat_w[001]
                   ,rdat_w[070]
                   ,rdat_w[075]
                   ,rdat_w[015]
                   ,rdat_w[088]
                   ,rdat_w[093]
                   ,rdat_w[083]
                   ,rdat_w[039]
                   ,rdat_w[117]
                   ,rdat_w[042]
                   ,rdat_w[117]
                   ,rdat_w[069]
                   ,rdat_w[093]
                   ,rdat_w[063]
                   ,rdat_w[037]
                   ,rdat_w[097]
                   ,rdat_w[055]
                   ,rdat_w[079]
                   ,rdat_w[022]
                   ,rdat_w[097]
                   ,rdat_w[088]
                   ,rdat_w[110]
                   ,rdat_w[060]
                   ,rdat_w[046]
                   ,rdat_w[104]
                   ,rdat_w[028]
                   ,rdat_w[052]
                   ,rdat_w[036]
                   ,rdat_w[105]
                   ,rdat_w[122]
                   ,rdat_w[073]
                   ,rdat_w[124]
                   ,rdat_w[123]
                   ,rdat_w[057]
                   ,rdat_w[048]
                   ,rdat_w[059]
                   ,rdat_w[030]
                   ,rdat_w[124]
                   ,rdat_w[122]
                   ,rdat_w[107]
                   ,rdat_w[083]
                   ,rdat_w[092]
                   ,rdat_w[041]
                   ,rdat_w[014]
                   ,rdat_w[094]
                   ,rdat_w[035]
                   ,rdat_w[108]
                   ,rdat_w[009]
                   ,rdat_w[067]
                   ,rdat_w[034]
                   ,rdat_w[033]
                   ,rdat_w[097]
                   ,rdat_w[127]
                   ,rdat_w[109]
                   ,rdat_w[122]
                   ,rdat_w[029]
                   ,rdat_w[075]
                   ,rdat_w[079]
                   ,rdat_w[053]
                   ,rdat_w[038]
                   ,rdat_w[067]
                   ,rdat_w[001]
                   ,rdat_w[019]
                   ,rdat_w[066]
                   ,rdat_w[008]
                   ,rdat_w[017]
                   ,rdat_w[056]
                   ,rdat_w[046]
                   ,rdat_w[009]
                   ,rdat_w[014]
                   ,rdat_w[062]
                   ,rdat_w[060]
                   ,rdat_w[061]
                   ,rdat_w[097]
                   ,rdat_w[050]
                   ,rdat_w[022]
                   ,rdat_w[055]
                   ,rdat_w[089]
                   ,rdat_w[008]
                   ,rdat_w[068]
                   ,rdat_w[104]
                   ,rdat_w[076]
                   ,rdat_w[089]
                   ,rdat_w[048]
                   ,rdat_w[006] };

assign  rk3_out = { rdat_w[014]
                   ,rdat_w[126]
                   ,rdat_w[011]
                   ,rdat_w[081]
                   ,rdat_w[074]
                   ,rdat_w[002]
                   ,rdat_w[044]
                   ,rdat_w[078]
                   ,rdat_w[053]
                   ,rdat_w[065]
                   ,rdat_w[009]
                   ,rdat_w[111]
                   ,rdat_w[056]
                   ,rdat_w[040]
                   ,rdat_w[104]
                   ,rdat_w[015]
                   ,rdat_w[112]
                   ,rdat_w[096]
                   ,rdat_w[114]
                   ,rdat_w[122]
                   ,rdat_w[106]
                   ,rdat_w[030]
                   ,rdat_w[123]
                   ,rdat_w[060]
                   ,rdat_w[000]
                   ,rdat_w[006]
                   ,rdat_w[045]
                   ,rdat_w[023]
                   ,rdat_w[071]
                   ,rdat_w[078]
                   ,rdat_w[091]
                   ,rdat_w[040]
                   ,rdat_w[111]
                   ,rdat_w[101]
                   ,rdat_w[070]
                   ,rdat_w[032]
                   ,rdat_w[016]
                   ,rdat_w[029]
                   ,rdat_w[102]
                   ,rdat_w[095]
                   ,rdat_w[025]
                   ,rdat_w[040]
                   ,rdat_w[040]
                   ,rdat_w[104]
                   ,rdat_w[011]
                   ,rdat_w[114]
                   ,rdat_w[061]
                   ,rdat_w[062]
                   ,rdat_w[097]
                   ,rdat_w[040]
                   ,rdat_w[093]
                   ,rdat_w[050]
                   ,rdat_w[049]
                   ,rdat_w[112]
                   ,rdat_w[114]
                   ,rdat_w[088]
                   ,rdat_w[005]
                   ,rdat_w[127]
                   ,rdat_w[072]
                   ,rdat_w[014]
                   ,rdat_w[090]
                   ,rdat_w[004]
                   ,rdat_w[079]
                   ,rdat_w[112]
                   ,rdat_w[036]
                   ,rdat_w[073]
                   ,rdat_w[075]
                   ,rdat_w[062]
                   ,rdat_w[082]
                   ,rdat_w[051]
                   ,rdat_w[096]
                   ,rdat_w[062]
                   ,rdat_w[023]
                   ,rdat_w[041]
                   ,rdat_w[000]
                   ,rdat_w[081]
                   ,rdat_w[069]
                   ,rdat_w[091]
                   ,rdat_w[103]
                   ,rdat_w[082]
                   ,rdat_w[029]
                   ,rdat_w[006]
                   ,rdat_w[074]
                   ,rdat_w[012]
                   ,rdat_w[001]
                   ,rdat_w[040]
                   ,rdat_w[091]
                   ,rdat_w[000]
                   ,rdat_w[107]
                   ,rdat_w[022]
                   ,rdat_w[081]
                   ,rdat_w[036]
                   ,rdat_w[066]
                   ,rdat_w[038]
                   ,rdat_w[045]
                   ,rdat_w[013]
                   ,rdat_w[069]
                   ,rdat_w[089]
                   ,rdat_w[027]
                   ,rdat_w[016]
                   ,rdat_w[073]
                   ,rdat_w[039]
                   ,rdat_w[011]
                   ,rdat_w[121]
                   ,rdat_w[123]
                   ,rdat_w[106]
                   ,rdat_w[079]
                   ,rdat_w[077]
                   ,rdat_w[082]
                   ,rdat_w[076]
                   ,rdat_w[054]
                   ,rdat_w[009]
                   ,rdat_w[002]
                   ,rdat_w[075]
                   ,rdat_w[063]
                   ,rdat_w[023]
                   ,rdat_w[113]
                   ,rdat_w[004]
                   ,rdat_w[017]
                   ,rdat_w[045]
                   ,rdat_w[002]
                   ,rdat_w[101]
                   ,rdat_w[043]
                   ,rdat_w[122]
                   ,rdat_w[030]
                   ,rdat_w[012]
                   ,rdat_w[105]
                   ,rdat_w[067] };

assign  rk4_out = rk1_out ^ rk2_out ^ rk3_out;

assign  rd1_out = { rdat_w[068]
                   ,rdat_w[016]
                   ,rdat_w[039]
                   ,rdat_w[060]
                   ,rdat_w[107]
                   ,rdat_w[061]
                   ,rdat_w[025]
                   ,rdat_w[028]
                   ,rdat_w[079]
                   ,rdat_w[007]
                   ,rdat_w[085]
                   ,rdat_w[102]
                   ,rdat_w[106]
                   ,rdat_w[041]
                   ,rdat_w[028]
                   ,rdat_w[061]
                   ,rdat_w[079]
                   ,rdat_w[106]
                   ,rdat_w[040]
                   ,rdat_w[101]
                   ,rdat_w[067]
                   ,rdat_w[074]
                   ,rdat_w[075]
                   ,rdat_w[095]
                   ,rdat_w[019]
                   ,rdat_w[101]
                   ,rdat_w[019]
                   ,rdat_w[091]
                   ,rdat_w[092]
                   ,rdat_w[085]
                   ,rdat_w[068]
                   ,rdat_w[045]
                   ,rdat_w[102]
                   ,rdat_w[027]
                   ,rdat_w[049]
                   ,rdat_w[087]
                   ,rdat_w[009]
                   ,rdat_w[029]
                   ,rdat_w[075]
                   ,rdat_w[022]
                   ,rdat_w[065]
                   ,rdat_w[064]
                   ,rdat_w[056]
                   ,rdat_w[031]
                   ,rdat_w[059]
                   ,rdat_w[095]
                   ,rdat_w[119]
                   ,rdat_w[103]
                   ,rdat_w[037]
                   ,rdat_w[004]
                   ,rdat_w[097]
                   ,rdat_w[101]
                   ,rdat_w[010]
                   ,rdat_w[089]
                   ,rdat_w[000]
                   ,rdat_w[115]
                   ,rdat_w[013]
                   ,rdat_w[011]
                   ,rdat_w[012]
                   ,rdat_w[069]
                   ,rdat_w[122]
                   ,rdat_w[052]
                   ,rdat_w[092]
                   ,rdat_w[083]
                   ,rdat_w[124]
                   ,rdat_w[091]
                   ,rdat_w[092]
                   ,rdat_w[055]
                   ,rdat_w[075]
                   ,rdat_w[094]
                   ,rdat_w[100]
                   ,rdat_w[071]
                   ,rdat_w[111]
                   ,rdat_w[050]
                   ,rdat_w[009]
                   ,rdat_w[061]
                   ,rdat_w[004]
                   ,rdat_w[049]
                   ,rdat_w[015]
                   ,rdat_w[071]
                   ,rdat_w[046]
                   ,rdat_w[088]
                   ,rdat_w[114]
                   ,rdat_w[114]
                   ,rdat_w[072]
                   ,rdat_w[091]
                   ,rdat_w[108]
                   ,rdat_w[102]
                   ,rdat_w[096]
                   ,rdat_w[067]
                   ,rdat_w[052]
                   ,rdat_w[122]
                   ,rdat_w[104]
                   ,rdat_w[087]
                   ,rdat_w[012]
                   ,rdat_w[017]
                   ,rdat_w[003]
                   ,rdat_w[054]
                   ,rdat_w[019]
                   ,rdat_w[113]
                   ,rdat_w[065]
                   ,rdat_w[050]
                   ,rdat_w[028]
                   ,rdat_w[015]
                   ,rdat_w[106]
                   ,rdat_w[047]
                   ,rdat_w[077]
                   ,rdat_w[045]
                   ,rdat_w[004]
                   ,rdat_w[052]
                   ,rdat_w[021]
                   ,rdat_w[071]
                   ,rdat_w[074]
                   ,rdat_w[037]
                   ,rdat_w[042]
                   ,rdat_w[015]
                   ,rdat_w[094]
                   ,rdat_w[013]
                   ,rdat_w[040]
                   ,rdat_w[056]
                   ,rdat_w[030]
                   ,rdat_w[080]
                   ,rdat_w[114]
                   ,rdat_w[020]
                   ,rdat_w[118]
                   ,rdat_w[069]
                   ,rdat_w[036]
                   ,rdat_w[076] };

assign  rd2_out = { rdat_w[107]
                   ,rdat_w[059]
                   ,rdat_w[017]
                   ,rdat_w[056]
                   ,rdat_w[040]
                   ,rdat_w[006]
                   ,rdat_w[070]
                   ,rdat_w[017]
                   ,rdat_w[090]
                   ,rdat_w[032]
                   ,rdat_w[028]
                   ,rdat_w[093]
                   ,rdat_w[084]
                   ,rdat_w[049]
                   ,rdat_w[102]
                   ,rdat_w[109]
                   ,rdat_w[098]
                   ,rdat_w[112]
                   ,rdat_w[089]
                   ,rdat_w[054]
                   ,rdat_w[029]
                   ,rdat_w[084]
                   ,rdat_w[052]
                   ,rdat_w[050]
                   ,rdat_w[009]
                   ,rdat_w[015]
                   ,rdat_w[021]
                   ,rdat_w[099]
                   ,rdat_w[039]
                   ,rdat_w[038]
                   ,rdat_w[126]
                   ,rdat_w[103]
                   ,rdat_w[074]
                   ,rdat_w[081]
                   ,rdat_w[040]
                   ,rdat_w[060]
                   ,rdat_w[006]
                   ,rdat_w[026]
                   ,rdat_w[036]
                   ,rdat_w[124]
                   ,rdat_w[119]
                   ,rdat_w[111]
                   ,rdat_w[111]
                   ,rdat_w[024]
                   ,rdat_w[082]
                   ,rdat_w[114]
                   ,rdat_w[094]
                   ,rdat_w[112]
                   ,rdat_w[000]
                   ,rdat_w[075]
                   ,rdat_w[022]
                   ,rdat_w[032]
                   ,rdat_w[120]
                   ,rdat_w[088]
                   ,rdat_w[017]
                   ,rdat_w[014]
                   ,rdat_w[006]
                   ,rdat_w[072]
                   ,rdat_w[086]
                   ,rdat_w[122]
                   ,rdat_w[077]
                   ,rdat_w[095]
                   ,rdat_w[122]
                   ,rdat_w[055]
                   ,rdat_w[070]
                   ,rdat_w[023]
                   ,rdat_w[033]
                   ,rdat_w[122]
                   ,rdat_w[012]
                   ,rdat_w[006]
                   ,rdat_w[069]
                   ,rdat_w[116]
                   ,rdat_w[048]
                   ,rdat_w[054]
                   ,rdat_w[103]
                   ,rdat_w[032]
                   ,rdat_w[053]
                   ,rdat_w[093]
                   ,rdat_w[015]
                   ,rdat_w[008]
                   ,rdat_w[023]
                   ,rdat_w[070]
                   ,rdat_w[110]
                   ,rdat_w[069]
                   ,rdat_w[034]
                   ,rdat_w[011]
                   ,rdat_w[111]
                   ,rdat_w[009]
                   ,rdat_w[019]
                   ,rdat_w[061]
                   ,rdat_w[081]
                   ,rdat_w[065]
                   ,rdat_w[073]
                   ,rdat_w[055]
                   ,rdat_w[077]
                   ,rdat_w[124]
                   ,rdat_w[077]
                   ,rdat_w[067]
                   ,rdat_w[120]
                   ,rdat_w[125]
                   ,rdat_w[083]
                   ,rdat_w[025]
                   ,rdat_w[004]
                   ,rdat_w[118]
                   ,rdat_w[079]
                   ,rdat_w[103]
                   ,rdat_w[120]
                   ,rdat_w[079]
                   ,rdat_w[015]
                   ,rdat_w[001]
                   ,rdat_w[027]
                   ,rdat_w[103]
                   ,rdat_w[075]
                   ,rdat_w[019]
                   ,rdat_w[121]
                   ,rdat_w[059]
                   ,rdat_w[004]
                   ,rdat_w[116]
                   ,rdat_w[061]
                   ,rdat_w[027]
                   ,rdat_w[086]
                   ,rdat_w[103]
                   ,rdat_w[045]
                   ,rdat_w[118]
                   ,rdat_w[090]
                   ,rdat_w[089]
                   ,rdat_w[061]
                   ,rdat_w[119] };

assign  rd3_out = { rdat_w[120]
                   ,rdat_w[040]
                   ,rdat_w[053]
                   ,rdat_w[084]
                   ,rdat_w[111]
                   ,rdat_w[055]
                   ,rdat_w[083]
                   ,rdat_w[099]
                   ,rdat_w[070]
                   ,rdat_w[049]
                   ,rdat_w[039]
                   ,rdat_w[049]
                   ,rdat_w[015]
                   ,rdat_w[075]
                   ,rdat_w[083]
                   ,rdat_w[117]
                   ,rdat_w[095]
                   ,rdat_w[055]
                   ,rdat_w[126]
                   ,rdat_w[032]
                   ,rdat_w[013]
                   ,rdat_w[058]
                   ,rdat_w[030]
                   ,rdat_w[073]
                   ,rdat_w[044]
                   ,rdat_w[090]
                   ,rdat_w[054]
                   ,rdat_w[024]
                   ,rdat_w[037]
                   ,rdat_w[018]
                   ,rdat_w[114]
                   ,rdat_w[106]
                   ,rdat_w[027]
                   ,rdat_w[003]
                   ,rdat_w[032]
                   ,rdat_w[094]
                   ,rdat_w[103]
                   ,rdat_w[025]
                   ,rdat_w[091]
                   ,rdat_w[105]
                   ,rdat_w[120]
                   ,rdat_w[112]
                   ,rdat_w[121]
                   ,rdat_w[094]
                   ,rdat_w[070]
                   ,rdat_w[105]
                   ,rdat_w[016]
                   ,rdat_w[045]
                   ,rdat_w[126]
                   ,rdat_w[009]
                   ,rdat_w[023]
                   ,rdat_w[112]
                   ,rdat_w[071]
                   ,rdat_w[014]
                   ,rdat_w[099]
                   ,rdat_w[068]
                   ,rdat_w[117]
                   ,rdat_w[039]
                   ,rdat_w[111]
                   ,rdat_w[123]
                   ,rdat_w[047]
                   ,rdat_w[033]
                   ,rdat_w[024]
                   ,rdat_w[017]
                   ,rdat_w[046]
                   ,rdat_w[114]
                   ,rdat_w[070]
                   ,rdat_w[030]
                   ,rdat_w[091]
                   ,rdat_w[100]
                   ,rdat_w[088]
                   ,rdat_w[049]
                   ,rdat_w[073]
                   ,rdat_w[070]
                   ,rdat_w[119]
                   ,rdat_w[067]
                   ,rdat_w[015]
                   ,rdat_w[087]
                   ,rdat_w[103]
                   ,rdat_w[044]
                   ,rdat_w[011]
                   ,rdat_w[030]
                   ,rdat_w[084]
                   ,rdat_w[024]
                   ,rdat_w[093]
                   ,rdat_w[072]
                   ,rdat_w[073]
                   ,rdat_w[075]
                   ,rdat_w[111]
                   ,rdat_w[095]
                   ,rdat_w[077]
                   ,rdat_w[043]
                   ,rdat_w[083]
                   ,rdat_w[112]
                   ,rdat_w[005]
                   ,rdat_w[056]
                   ,rdat_w[096]
                   ,rdat_w[052]
                   ,rdat_w[100]
                   ,rdat_w[051]
                   ,rdat_w[046]
                   ,rdat_w[096]
                   ,rdat_w[024]
                   ,rdat_w[044]
                   ,rdat_w[007]
                   ,rdat_w[108]
                   ,rdat_w[065]
                   ,rdat_w[085]
                   ,rdat_w[093]
                   ,rdat_w[077]
                   ,rdat_w[078]
                   ,rdat_w[063]
                   ,rdat_w[050]
                   ,rdat_w[017]
                   ,rdat_w[019]
                   ,rdat_w[057]
                   ,rdat_w[087]
                   ,rdat_w[014]
                   ,rdat_w[113]
                   ,rdat_w[073]
                   ,rdat_w[054]
                   ,rdat_w[006]
                   ,rdat_w[102]
                   ,rdat_w[108]
                   ,rdat_w[110]
                   ,rdat_w[094]
                   ,rdat_w[050]
                   ,rdat_w[108] };

assign  rd4_out = rd1_out ^ rd2_out ^ rd3_out;


endmodule
